
#ifndef STACK_H_INCLUDED
#define STACK_H_INCLUDED

/* *************************************************************** Strukturen */

/* anonyme Strukturen verhindern den Zugriff auf interne Daten von außerhalb */
typedef struct sStack stack;

/**@brief Typedef für eine Funktion, die ein Datenelement ausgibt.
 */
typedef void(*fStackPrinter)(const void* datum);

/* ***************************************************************** Methoden */

/**@brief Liefert einen Zeiger auf einen neuen Stack.
 */
extern stack* newStack();

/**@brief Gibt den belegten Speicher des Stacks wieder frei.
 * @param s   Stack
 */
extern void deleteStack(stack* s);

/**@brief Legt ein Element auf den Stack.
 * @param s   Stack
 * @param i   Wert
 */
extern void stackPush(stack* s, void* i);

/**@brief Gibt das oberste Element des Stacks zurück, ohne ihn vom Stack zu
 * löschen.
 * @param s   Stack
 * @return oberster Wert
 */
extern void* stackTop(const stack* s);

/**@brief Gibt das oberste Element des Stacks zurück und nimmt es vom Stack.
 * @param s   Stack
 * @return oberster Wert
 */
extern void* stackPop(stack* s);

/**@brief Liefert als Ergebnis einen Wert der besagt, ob der Stack leer ist.
 * @param s   Stack
 * @return 0 => Stack ist nicht leer \n
 *         1 => Stack ist leer
 */
extern int stackIsEmpty(const stack* s);

/**@brief Gibt den Stack vom zuerst eingefügten bis zum zuletzt eingefügten
 * Element aus.
 * @param s      Stack
 * @param print  Funktion, die ein Element ausgibt
 */
extern void stackPrint(const stack* s, fStackPrinter print);

#endif
